unit Automaton;                     

{ **************************************************************************** }
{ *                                                                          * }
{ *                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

interface
uses
  Classes;
type
  TEvent = integer;      //     "".
  TState = integer;      //     " ".
type

{ **************************************************************************** }
{ *                                                                          * }
{ *   : TAutomaton.                                            * }
{ *  :       ,    .        * }
{ *                                                                          * }
{ **************************************************************************** }

  TAutomatonsDispatcher = class;
  TAutomaton = class
  private
    IsRunning:    boolean;    // ,     .
  protected
    Name:           integer;  //  .
    Description:    string;   //   .
    State:          TState;   //   .
  public
    LogInput:       boolean;  //   .
    LogOutput:      boolean;  //   .
    LogWake:        boolean;  //    .
    LogStateChange: boolean;  //    .
    LogSleep:       boolean;  //    .
    LogError:       boolean;  //     .
  public
    property y: TState read State; //     .
  public
    atmDispatcher: TAutomatonsDispatcher; //   
  public
    constructor Init(AutomatonName: integer; AutomatonDescription: string);
    procedure   Wake(Event: TEvent);
  protected
    procedure HandleState(Event: TEvent);    virtual;
    procedure EnterState();                  virtual;
    procedure Log(AutomatonMessage: string); virtual;
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *   : TAutomatonsDispatcher.                                 * }
{ *  :            * }
{ *                   ,    ,       * }
{ *                       ..                    * }
{ *                                                                          * }
{ **************************************************************************** }

  TAutomatonsDispatcher = class
  public
    Automatons : array of TAutomaton; //    .
    z: TStrings;                      //    .
    x: TStrings;                      //    .
    y: array of TStrings;             //    .
    e: TStrings;                      //    .
    constructor Create();
    destructor  Free();
    procedure   AddAutomaton (A: TAutomaton);
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

implementation
uses
  SysUtils;

{ **************************************************************************** }
{ *                                                                          * }
{ *  :           TAutomaton.                                            * }
{ *   : Init.                                                  * }
{ *  :        TAutomaton.                         * }
{ *                                                                          * }
{ *  :       AutomatonName -  .                        * }
{ *                   AutomatonDescription -   .    * }
{ *                                                                          * }
{ **************************************************************************** }

  constructor TAutomaton.Init
  (
    AutomatonName:        integer;
    AutomatonDescription: string
  );
  begin
    Name        := AutomatonName;
    Description := AutomatonDescription;
    State       := 0;
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *  :           TAutomaton.                                            * }
{ *   : Wake.                                                  * }
{ *  :           .      * }
{ *                                                                          * }
{ *  :       Event - ,     . * }
{ *                                                                          * }
{ **************************************************************************** }

  procedure TAutomaton.Wake
  (
    Event: TEvent
  );
  var
    OldState: TState;
  begin
    if LogWake then
      Log(' A' + IntToStr(Name) + ' "' + Description +
          '"    ' + IntToStr(State) + ' "'
                                   + atmDispatcher.y[Name][State] +
          '"   e' + IntToStr(Event) + ' "' +
                             atmDispatcher.e[Event] + '"');
    if (IsRunning) then
      Log('    A' + IntToStr(Name) + ' "'
                                                  + Description +
          '"   ' + IntToStr(State) + ' "'
                           + atmDispatcher.y[Name][State]  + '"' +
          '   e' + IntToStr(Event) + ' "'
                          + atmDispatcher.e[Event] + '"');
    IsRunning := true;
    OldState := State;
    HandleState(Event);
    if (OldState <> State) then
    begin
      if LogStateChange then
        Log(' A' + IntToStr(Name)  + ' "' + Description +
            '"    ' + IntToStr(OldState) +
            '   ' + IntToStr(State));
      EnterState();
    end;
    IsRunning := false;
    if LogSleep then
      Log(' A' + IntToStr(Name)  + ' "' + Description +
          '"    e' + IntToStr(Event)  + ' "'
                                           + atmDispatcher.e[Event] +
          '"   ' + IntToStr(State) + ' "'
                           + atmDispatcher.y[Name][State] + '"');
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *  :           TAutomaton.                                            * }
{ *   : HandleState.                                           * }
{ *  :      ,   .  ,   * }
{ *                          * }
{ *                   .    "switch".     * }
{ *                                                                          * }
{ *  :       Event - ,    .             * }
{ *                                                                          * }
{ **************************************************************************** }

  procedure TAutomaton.HandleState(Event: TEvent);
  begin
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *  :           TAutomaton.                                            * }
{ *   : EnterState.                                            * }
{ *  :      ,   .  ,   * }
{ *                            * }
{ *                    .          * }
{ *                   "switch"                                               * }
{ *                                                                          * }
{ **************************************************************************** }

  procedure TAutomaton.EnterState();
  begin
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *  :           TAutomaton.                                            * }
{ *   : Log.                                                   * }
{ *  :      ,   .       * }
{ *                      .                         * }
{ *                                                                          * }
{ *  :       AutomatonMessage - ,     * }
{ *                                                                  * }
{ *                                                                          * }
{ **************************************************************************** }

  procedure TAutomaton.Log(AutomatonMessage: string);
  begin
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *  :           TAutomatonsDispatcher.                                 * }
{ *   : Create.                                                * }
{ *  :        TAutomatonsDispatcher.              * }
{ *                                                                          * }
{ **************************************************************************** }

  constructor TAutomatonsDispatcher.Create;
  begin
    x := TStringList.Create();
    z := TStringList.Create();
    e := TStringList.Create();
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *  :           TAutomatonsDispatcher.                                 * }
{ *   : Create.                                                * }
{ *  :        TAutomatonsDispatcher.               * }
{ *                                                                          * }
{ **************************************************************************** }

  destructor TAutomatonsDispatcher.Free;
  var
   i: integer;
  begin
    x.Free;
    z.Free;
    e.Free;
    for i:=0 to High(y) do
      y[i].Free;
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *  :           TAutomatonsDispatcher.                                 * }
{ *   : AddAutomaton.                                          * }
{ *  :           .           * }
{ *                                                                          * }
{ *  :       A - ,           * }
{ *                                                                * }
{ *                                                                          * }
{ **************************************************************************** }

  procedure TAutomatonsDispatcher.AddAutomaton (A: TAutomaton);
  begin
    if Automatons = nil then
    begin
      SetLength(Automatons,1);
      SetLength(y,1)
    end else
    begin
      SetLength(Automatons, High(Automatons)+2);
      SetLength(y, High(y)+2);
    end;

    Automatons[High(Automatons)] := A;
    A.atmDispatcher := self;
    y[High(y)] :=  TStringList.Create;
  end;

{ **************************************************************************** }
{ *                                                                          * }
{ *    "Automaton.pas"                                            * }
{ *                                                                          * }
{ **************************************************************************** }

end.


